/*
 * TaRyFikator by Maciej Korze <eaquer@ceti.pl>
 *
 * $Id: trfs.c,v 1.35 2003/02/02 18:32:03 eaquer Exp $
 * 
 * Copyright (C) 2002  Maciej Korze
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675
 * Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

/*
 * Ze wzgldy na "dziwn" implementacj strftime i strptime w Linuksie,
 * trzeba si dostosowa.
 */
#ifdef LINUX
#define TIME_CHAR	"%c"
#endif

#ifdef FREEBSD
#define	TIME_CHAR	"%+"
#endif

#define VERSION	"0.4"

/* Janiejszy tekst do drukowania obramowa. */
#define BOLD "\033[1;37m"
#define NORMAL "\033[0m"

/*
 * Do okrelenia z ktrego pliku mamy odczytywa informacje o poczeniach.
 * (Patrz funkcja pobierz()).
 */
u_int             plik;

/*
 * Wywietla podany tekst rozjanion czcionk. Uywane do rysowania ramek.
 */
void            bold(const char tekst[]);

/* Oblicza koszt poczenia, ilo impulsw, etc. */
void            oblicz(u_int data, const u_int koniec, u_int *sumimpulsy, double *sumkoszt, char *flagi, const size_t rozm);

/*
 * Wywietla numer wersji programu.
 */
void            version(void);

/*
 * Wywielta list moliwych do uycia argumentw.
 */
void            usage(void);

/*
 * Kolejne wywoania zwracaj zawarto kolejnych wierszy z pliku fd1. Jeeli
 * fd1 si skoczy (EOF), to zwracana jest zawarto z pliku fd2. Dopiero gdy
 * fd2 si skoczy jest zwracany NULL. Odczytane linie s zapisywane do bufora.
 * Jednorazowo jest odczytywanych maksymalnie 'lim' znakw.
 * fd1 musi istnie. Jeeli fd2 nie istnieje, to fd2 musi by rwne NULL.
 */
char           *pobierz(FILE * fd1, FILE * fd2, char *bufor, const int lim);

/*
 * Sprawdza czy dany 'text' zawiera dozwolone znaki. Lista dozwolonych znakw
 * jest zapisana w tablicy allowed.
 */
u_int             validate(const char text[], const char *allowed);

/*
 * Oblicza od ktrego znaku w podanym buforze zaczyna si nastpna data.
 * Umoliwia poprawne odczytanie dat z rnych stref czasowych, gdy nawy tych
 * stref maj rn ilo znakw.
 */
u_int		  next_date_start(const char date[]);

int
main(int argc, char *argv[])
{
	/*
	 * Lista dozwolonych znakw w wartociach liczbowych.
	 */
	char            allowed_num[] =
	{
		'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '\0'
	};

	/*
	 * Lista znakw dozwolonych w nazwach plikw, katalogw.
	 */
	char            allowed_path[] =
	{
		' ', '!', '&', '\'', '(', ')', '+', ',', '-', '.', '/', '0',
		'1', '2', '3', '4', '5', '6', '7', '8', '9', ':', '?', '@',
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
		'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
		'Y', 'Z', '[', ']', '_', '`', 'a', 'b', 'c', 'd', 'e', 'f',
		'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r',
		's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '}', '~', '\0'
	};

	/*
	 * Lista znakw dozwolonych w stringach okrelajcych dat.
	 */
	char            allowed_date[] =
	{
		' ', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ':',
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
		'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
		'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
		'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
		'w', 'x', 'y', 'z', '\n', '\0'
	};

	FILE           *fd1, *fd2;

	/*
	 * Do przechowywania rnych "mieci".
	 */
	char            c, bufor[200];
	int		i = 0;

	/*
	 * Do przechowywania 'R', 'Z' lub 'P' ;-).
	 */
	char		status;

	/*
	 * Do przechowywania cieki do pliku z list pocze.
	 */
	char           *file_log;

	/*
	 * Do przechowywania nazwy pliku z list pocze.
	 */
	char	       *file_log_suffix;

	/*
	 * Domylna nazwa pliku z list pocze.
	 */
	char            file_log_suffix_default[] = ".trf";

	/*
	 * Do przechowywania nazwy pliku z informacj o aktualnym poczeniu.
	 */
	char	       *file_current;

	/*
	 * Kocwka nazwy pliku z informacj o aktualnym poczeniu.
	 */
	char            file_current_suffix[] = ".current";
	
	/*
	 * Do przechowywania zawartoci TRFLIMIT.
	 */
	char	       *trflimit;
	
	/*
	 * Do przechowywania zawartoci HOME.
	 */
	char           *home;

	/*
	 * Do przechowywania 'flag' obrazujcych przebieg klasyfikacji
	 * poczenia.
	 */
	char	       *flagi;

	/*
	 * Sumaryczny koszt pocze w danym miesicu.
	 */
	double          sumkoszt = 0.00;

	/*
	 * Dugo ryczatu w _sekundach_.
	 * 81000 sekund = 22,5 godziny.
	 */
	int             limit = 81000;

	/*
	 * Numer biecego miesica.
	 */
	u_int           month = 0;

	/*
	 * Dugo danego poczenia (w sekundach).
	 */
	int             dlugosc;

	/*
	 * Sumaryczna dugo pocze.
	 */
	u_int		sumdlugosc = 0;

	/*
	 * Przed zmian wartoci 'sumdlugosc', aktualna warto jest zapisywana
	 * do 'poprzsumdlugosc'.
	 */
	u_int		poprzsumdlugosc = 0;

	/*
	 * Ilo sekund jaka upyna od Epoch w momencie rozpoczcia
	 * poczenia.
	 */
	int             sekundyS;

	/*
	 * Ilo sekund jaka upyna od Epoch w momencie zakoczenia
	 * poczenia.
	 */
	int		sekundyK;

	/*
	 * Do przechowywania iloci sekund, minut i godzin.
	 */	
	u_int             hou, min, sec;

	/*
	 * Rozmiary rnych char'w. Nazwy odpowiednikw poprzedzone s 'rozm_'.
	 */
	size_t          rozm_home, rozm_file_log_suffix, rozm_file_log;
	size_t          rozm_file_current, rozm_trflimit, rozm_flagi;

	/*
	 * Struktury do przechowywania dat: rozpoczcia poczenia, koca
	 * poczenia i aktualnej daty.
	 */
	struct tm       stime, etime, atime;

	/*
	 * Przechowuje dat w postaci liczby sekund od Epoch.
	 */
	time_t          atimet;

	/*
	 * Liczba linii w pliku ~/.trf i liczba linii odczytanych.
	 */
	u_int	    linie = 0, odczytane_linie = 0;

	/*
	 * Sumaryczna liczba impulsw.
	 */
	u_int		sumimpulsy = 0;

	/*
	 * Liczba pocze dokonanych w danym miesicu.
	 */
	u_int		liczbapolaczen = 0;

	/*
	 * Z ktrego pliku mamy czyta informacje o poczeniach.
	 * 1 -> czytaj z ~/.trf
	 * 2 -> czytaj z ~/.trf.current
	 */
	plik = 1;

	/*
	 * Rezerwujemy pami dla 'flagi'.
	 */
	rozm_flagi = 9;
	flagi = malloc(rozm_flagi);

	/*
	 * Inicjujemy struktur 'atimet' aktualn dat.
	 */
	(void)time(&atimet);

	/*
	 * Do atime zapisujemy aktualn dat.
	 */
	atime = *localtime(&atimet);

	/*
	 * Do month zapisujemy numer biecego miesica.
	 */
	month = (u_int)atime.tm_mon;

	/*
	 * Jeeli zmienna TRFLIMIT jest okrelona, to ...
	 */
	if (getenv("TRFLIMIT") != NULL) {
		/*
		 * Sprawdzamy jaki rozmiar ma TRFLIMIT.
		 */
		rozm_trflimit = strlen(getenv("TRFLIMIT")) + 1;
		/*
		 * I wanie tak ilo miejsca przygotowujemy do przechowywania
		 * wartoci.
		 */
		trflimit = malloc(rozm_trflimit);
		/*
		 * Jeeli malloc si nie popisa, to wychodzimy.
		 */
		if (!trflimit) {
			fprintf(stderr, "Nie udao si zalokowa pamici dla 'trflimit'.\n");
			exit(1);
		}
		/*
		 * Kopiujemy warto TRFLIMIT do 'trflimit'.
		 */
		strncpy(trflimit, getenv("TRFLIMIT"), rozm_trflimit);
		trflimit[rozm_trflimit] = '\0';

		/*
		 * Jeeli w 'trflimit' s jakie paskudztwa to wychodzimy.
		 */
		if (validate(trflimit, allowed_num) == 1) {
			fprintf(stderr, "Zmienna TRFLIMIT zawiera nieprawidowe znaki. Dozwolone s tylko cyfry.\n");
			exit(1);
		}
		/*
		 * Jeeli wszysko jest O.K., to konwertujemy warto ze stringa
		 * na int.
		 */
		limit = atoi(trflimit);
		/*
		 * Zwalniamy ju niepotrzebn pami.
		 */
		free(trflimit);
	}

	/*
	 * Sprawdzamy czy limit ma dodatni warto.
	 */
	if (limit < 0) {
		fprintf(stderr, "Warto limitu musi by wiksza od zera.\n");
		exit(1);
	}

	/*
	 * Aby okreli pooenia pliku z logiem musimy zna warto zmiennej
	 * HOME.
	 */
	if (getenv("HOME") == NULL) {
		fprintf(stderr, "Okrel zmienn rodowiska HOME.\n");
		exit(1);
	}

	/*
	 * Sprawdzamy jaki rozmiar ma HOME.
	 */
	rozm_home = strlen(getenv("HOME")) + 1;

	/*
	 * Alokujemy pami dla home, etc...
	 */
	home = malloc(rozm_home);
	if (!home) {
		fprintf(stderr, "Nie udao si zalokowa pamici dla 'home'.\n");
		exit(1);
	}

	/*
	 * Zapisujemy warto HOME do 'home'.
	 */
	strncpy(home, getenv("HOME"), rozm_home);
	home[rozm_home] = '\0';

	/*
	 * Sprawdzamy czy 'home' zawiera tylko dozwolone znaki.
	 */
	if (validate(home, allowed_path) == 1) {
		fprintf(stderr, "Zmienna HOME zawiera niedzowolone znaki.\n");
		exit(1);
	}

	/*
	 * Jeeli zmienna TRFFILE jest okrelona, to...
	 */
	if (getenv("TRFFILE") != NULL) {
		/*
		 * Sprawdzamy jej rozmiar, alokujemy dla niej pami, zapisujemy
		 * warot do 'file_log_suffix' i sprawdzamy czy zawiera tylko
		 * dozwolone znaki.
		 */
		rozm_file_log_suffix = strlen(getenv("TRFFILE")) + 1;
		file_log_suffix = malloc(rozm_file_log_suffix);
		if (!file_log_suffix) {
			fprintf(stderr, "Nie udao si zalokowa pamici dla 'file_log_suffix'.\n");
			exit(1);
		}
		strncpy(file_log_suffix, getenv("TRFFILE"), rozm_file_log_suffix);
		file_log_suffix[rozm_file_log_suffix] = '\0';
		if (validate(file_log_suffix, allowed_path) == 1) {
			fprintf(stderr, "Zmienna TRFFILE zawiera niedozwolone znaki.\n");
			exit(1);
		}
	} else {
		/*
		 * Jeeli zmienna TRFFILE nie jest okrelona, to uywamy wartoci
		 * domylnych...
		 */
		rozm_file_log_suffix = strlen(file_log_suffix_default) + 1;
		file_log_suffix = malloc(rozm_file_log_suffix);
		if (!file_log_suffix) {
			fprintf(stderr, "Nie udao si zalokowa pamici dla 'file_log_suffix' (2).\n");
			exit(1);
		}
		strncpy(file_log_suffix, file_log_suffix_default, rozm_file_log_suffix);
		file_log_suffix[rozm_file_log_suffix] = '\0';
	}

	/*
	 * Sprawdzamy jaki rozmiar mam mie 'file_log' (rozmiar sufiksu +
	 * rozmiar 'home').
	 */
	rozm_file_log = rozm_file_log_suffix + rozm_home;

	/*
	 * Alokujemy miejsce dla 'file_log'.
	 */
	file_log = malloc(rozm_file_log);
	if (!file_log) {
		fprintf(stderr, "Nie udao si zalokowa pamici dla 'file_log'.\n");
		exit(1);
	}

	/*
	 * Zapisujemy w 'file_log' ostateczn posta cieki dostpu do pliku z
	 * logiem: 'home' + '/' + sufiks.
	 */
	strncpy(file_log, home, rozm_file_log);
	file_log[rozm_file_log] = '\0';
	strncat(file_log, "/", rozm_file_log - strlen(file_log));
	strncat(file_log, file_log_suffix, rozm_file_log - strlen(file_log));

	/*
	 * Zwalniamy pami zajmowan przez 'home' i 'file_log_suffix'. Te
	 * zmienne nie bd ju nam potrzebne.
	 */
	free(home);
	free(file_log_suffix);

	/*
	 * Okrelamy rozmiar 'file_current': rozmiar 'file_log' - znak
	 * terminacji ('\0') + rozmiar 'current_suffix'.
	 */
	rozm_file_current = rozm_file_log - 1 + strlen(file_current_suffix);

	/*
	 * Alokujemy pami dla 'file_current'.
	 */
	file_current = malloc(rozm_file_current);
	if (!file_current) {
		fprintf(stderr, "Nie udao si zalokowa pamici dla 'file_current'.\n");
		exit(1);
	}

	/*
	 * W 'file_current' zapisujemy ostateczn posta cieki dostpu do
	 * pliku z informacj o aktualnym poczeniu.
	 */
	strncpy(file_current, file_log, rozm_file_current);
	file_current[rozm_file_current] = '\0';
	strncat(file_current, file_current_suffix, rozm_file_current - strlen(file_current));

	/*
	 * Sprawdzamy z jakimi argumentami zosta uruchomiony program.
	 */
	while ((i = getopt(argc, argv, "Vm:h")) != -1) {
		switch (i) {
		case 'V':
			/*
			 * Jeeli jednym z argumentw jest '-V', to wywietlamy
			 * informacje o wersji programu i wychodzimy.
			 */
			version();
			exit(0);
		case 'm':
			/*
			 * Jeeli argumentem jest '-m N', to ...
			 * ... sprawdzamy czy podana liczba skada si tylko z
			 * cyfr.
			 */
			if (validate(optarg, allowed_num) == 1) {
				fprintf(stderr, "Podano zy numer miesica. Dozwolone s tylko cyfry.\n");
				exit(1);
			}
			/*
			 * Konwertujemy stringa do int.
			 */
			month = (u_int)atoi(optarg);
			month--;
			/*
			 * Sprawdzamy czy warto 'month' znajduje si w
			 * dozwolonym przedziale.
			 */
			if (month > 11) {
				fprintf(stderr, "Podano zy numer miesica.\n");
				exit(1);
			}
			break;
		case '?':
		default:
			/*
			 * Jeeli program zosta wywoany z dowoln inn opcj,
			 * to wywietlamy informacje o moliwych argumentach i
			 * wychodzimy.
			 */
			usage();
			exit(1);
		}
	}

	/*
	 * Jeeli plik z informacjami o poczeniach nie istnieje, to
	 * wychodzmimy.
	 */
	if ((fd1 = fopen(file_log, "r")) == NULL) {
		fprintf(stderr, "Brak pliku z logami.\n");
		exit(1);
	}

	/*
	 * Sprawdzamy ile linii ma plik z logami.
	 */
	while ((c = getc(fd1)) != EOF) {
		if (c == '\n')
			linie++;
	}

	/* ...i zamykamy go. */
	(void)fclose(fd1);

	/* Wywietlamy nagwek ramki. */
	bold(",--------------------------------+----------+----------+----------+-----------.\n");
	bold("|");
	printf(" pocztek - koniec poczenia   ");
	bold("|");
	printf(" dugo  ");
	bold("|");
	printf(" impulsy  ");
	bold("|");
	printf(" koszt    ");
	bold("|");
	printf(" flagi     ");
	bold("|\n");
	bold("+--------------------------------+----------+----------+----------+-----------+\n");

	/*
	 * Otwieramy pliki ~/.trf i ~/.trf.current i zwalniamy pami zajmowan
	 * przez stringi zawierajce nazwy tych plikw.
	 */
	fd1 = fopen(file_log, "r");
	free(file_log);
	fd2 = fopen(file_current, "r");
	free(file_current);

	/*
	 * Dopki bdziemy mogli pobra jakie dane z plikw, to...
	 */
	while (pobierz(fd1, fd2, bufor, 63) != NULL) {
		/*
		 * Zwikszamy liczb odczytanych linii.
		 */
		odczytane_linie++;

		/*
		 * Sprawdzamy, czy bufor zawiera tylko dozwolone znaki.
		 */
		if (validate(bufor, allowed_date) == 1) {
			fprintf(stderr, NORMAL);
			fprintf(stderr, "Rekord znajdujcy si w %u linii zawiera nieprawidowe znaki. Dozwolone s tylko cyfry, mae i due litery oraz dwukropek.\n", odczytane_linie);
			exit(1);
		}

		/*
		 * Do struktury 'stime' zapisujemy pierwsz dat odczytan z bufora.
		 */
		if (strptime(bufor, TIME_CHAR, &stime) == NULL) {
			fprintf(stderr, NORMAL);
			fprintf(stderr, "Bd podczas przetwarzania rekordu z %u linii.\n", odczytane_linie);
			exit(1);
		}

		/*
		 * Do struktury 'etime' zapisujemy drug dat znajdujc si w
		 * buforze.
		 */
		if (strptime(&(bufor[(int) next_date_start(bufor)]), TIME_CHAR, &etime) == NULL) {
			fprintf(stderr, NORMAL);
			fprintf(stderr, "Bd podczas przetwarzania rekordu z %u linii.\n", odczytane_linie);
			exit(1);
		}

		/*
		 * Jeeli miesic w ktrym si rozpoczo poczenie, jest inny
		 * ni zawarty w zmiennej 'mont', to pomijamy to poczenie.
		 */
		if (month != (u_int)stime.tm_mon)
			continue;

		/*
		 * Zwikszamy liczb pocze, ktre zostay dokonane w danym
		 * miesicu.
		 */
		liczbapolaczen++;

		/*
		 * Wywietlamy informacje o czasie rozpoczcia i zakoczenia
		 * poczenia.
		 */
		bold("|");
		printf(" %i/%2i/%2i ", stime.tm_year + 1900, stime.tm_mon + 1, stime.tm_mday);

		if (stime.tm_hour < 10)
			printf("0");
		printf("%i:", stime.tm_hour);
		if (stime.tm_min < 10)
			printf("0");
		printf("%i:", stime.tm_min);
		if (stime.tm_sec < 10)
			printf("0");
		printf("%i - ", stime.tm_sec);

		if (etime.tm_hour < 10)
			printf("0");
		printf("%i:", etime.tm_hour);
		if (etime.tm_min < 10)
			printf("0");
		printf("%i:", etime.tm_min);
		if (etime.tm_sec < 10)
			printf("0");
		printf("%i ", etime.tm_sec);

		bold("|");

		/*
		 * Dat ze struktury 'stime' zamieniamy na licz sekund od
		 * Epoch i zapisujemy w 'sekundyS'.
		 */
		if (strftime(bufor, sizeof(bufor) - 1, "%s", &stime) == 0) {
			fprintf(stderr, "\nWystpi bd funkcji strftime() podczas przetwarzania rekordu z linii numer %u.\n", odczytane_linie);
			exit(1);
		}
		sekundyS = atoi(bufor);

		/*
		 * Jeeli funkcja strftime() co namieszaa, to wychodzimy...
		 */
		if (sekundyS <= 0) {
			fprintf(stderr, "\nFunkcja strftime zwrcia z liczb sekund. Zgo ten bd.\n");
			exit(1);
		}

		/*
		 * Dat ze struktury 'etime' zamieniamy na liczb sekund od
		 * Epoch i zapisujemy w 'sekundyK'.
		 */
		if (strftime(bufor, sizeof(bufor) - 1, "%s", &etime) == 0) {
			fprintf(stderr, "\nWystpi bd funkcji strftime() podczas przetwarzania rekordu z linii numer %u.\n", odczytane_linie);
			exit(1);
		}
		sekundyK = atoi(bufor);

		if (sekundyK <= 0) {
			fprintf(stderr, "\nFunkcja strftime() zwrcia z liczb sekund.\n");
			exit(1);
		}

		/*
		 * Obliczamy dugo poczenia w sekundach.
		 */
		dlugosc = sekundyK - sekundyS;

		/*
		 * Jeeli dugo poczenia jest ujemna, to co musi by nie
		 * tak.
		 */
		if (dlugosc < 0) {
			fprintf(stdout, "\n");
			fprintf(stderr, NORMAL "Bd. Ujemna dugo poczenia.\n");
			exit(1);
		}

		/*
		 * Dotychczasow warto 'sumdlugosc' zapisujemy do
		 * 'poprzsumdlugosc', a 'sumdlugosc' zwikszamy o czas
		 * aktualnego poczenia.
		 */
		poprzsumdlugosc = sumdlugosc;
		sumdlugosc += (u_int)dlugosc;

		/*
		 * Dugo poczenia w sekundach zamieniamy na normaln dat i
		 * wywietlamy.
		 */
		sec = (u_int)dlugosc;
		min = 0;
		hou = 0;

		while (sec >= 60) {
			min = min + (sec / 60);
			sec = sec % 60;
		}

		while (min >= 60) {
			hou = hou + (min / 60);
			min = min % 60;
		}

		if (hou < 10) {
			printf(" 0");
			printf("%u:", hou);
		} else {
			printf("%2u:", hou);
		}

		if (min < 10)
			printf("0");
		printf("%u:", min);

		if (sec < 10)
			printf("0");
		printf("%u ", sec);

		bold("|");

		if ((int)sumdlugosc < limit && (int)poprzsumdlugosc < limit) {
			/*
			 * Jeeli nie przekroczylimy jeszcze czasu ryczatu, to
			 * nie musimy oblicza nic wicej.
			 */
			printf("          ");
			bold("|");
			printf("          ");
			bold("|");
			status = 'R';
		} else if ((int)poprzsumdlugosc < limit && (int)sumdlugosc > limit) {
			/*
			 * Jeeli poczenie zostao rozpoczte w czasie trwania
			 * ryczatu, a zakoczone jak przekroczylimy limit, to
			 * obliczamy ile zapacimy za czas poczenia dokonanego
			 * poza ryczatem.
			 */
			dlugosc = (int)sumdlugosc - (int)limit;
			oblicz((u_int)sekundyS + (u_int)limit - poprzsumdlugosc, (u_int)sekundyK, &sumimpulsy, &sumkoszt, flagi, rozm_flagi);
			bold("|");
			status = 'Z';
		} else if ((int)poprzsumdlugosc >= limit && (int)sumdlugosc >= limit) {
			/*
			 * Jeeli poczenie zostao dokonane cakowicie poza
			 * ryczatem, to tylko obliczamy jego koszt.
			 */
			oblicz((u_int)sekundyS, (u_int)sekundyK, &sumimpulsy, &sumkoszt, flagi, rozm_flagi);
			bold("|");
			status = 'P';
		} else {
			/*
			 * Ten fragment nie powinien zosta wykonany.
			 */
			fprintf(stderr, "\nBd. Poczenie nie zostao poprawnie zakwalifikowane.\n");
			exit(1);
		}

		/*
		 * Jeeli odczytalimy o jedn lini wicej ni liczba linii
		 * znajdujcych si w pliku ~/.trf, to znaczy, e ta linia jest
		 * z ~/.trf.current.
		 */
		if (odczytane_linie == (linie + 1)) {
			printf("%8s%cA", flagi, status);
		} else {
			printf("%9s%c", flagi, status);
		}
		bold(" |\n");
	}

	/*
	 * Zamykamy niepotrzebne ju pliki.
	 */
	(void)fclose(fd1);
	if (fd2) {
		(void)fclose(fd2);
	}

	/*
	 * Zwalniamy pami zajmowan przez 'flagi'.
	 */
	free(flagi);

	/*
	 * Jeeli liczba pocze jest rwna zero, to wywietlamy tylko
	 * niezbdn informacj i wychodzimy.
	 */
	if (liczbapolaczen == 0) {
		bold("|");
		printf(" Brak dokonanych pocze                                                     ");
		bold("|\n");
		bold("`--------------------------------+----------+----------+----------+-----------'\n");
		return 0;
	}

	/*
	 * Wywietlamy doln cz tabeli.
	 */
	bold("`--------------------------------+----------+----------+----------+-----------'\n");
	printf(" R - ryczat / Z - cz w ryczacie, cz poza / P - poza ryczatem\n");
	printf(" d - drosza taryfa / t - tasza / w - weekend / A - aktualne poczenie\n");
	printf(" ! - za dua liczba flag, nie wszystkie si zmieciy\n");

	/*
	 * Wywietlamy sumaryczny czas dokonanych pocze i inne "bajery".
	 */
	sec = sumdlugosc;
	min = 0;
	hou = 0;
	while (sec >= 60) {
		min = min + (sec / 60);
		sec = sec % 60;
	}
	while (min >= 60) {
		hou = hou + (min / 60);
		min = min % 60;
	}
	printf("\nSumaryczna dugo pocze:\t");
	if (hou < 10)
		printf("0");
	printf("%u:", hou);
	if (min < 10)
		printf("0");
	printf("%u:", min);
	if (sec < 10)
		printf("0");
	printf("%u\n", sec);

	printf("Sumaryczny koszt pocze:\t%.2f\n", sumkoszt);
	printf("Sumaryczna liczba impulsw:\t%u\n", sumimpulsy);
	printf("Liczba dokonanych pocze:\t%u\n", liczbapolaczen);

	/*
	 * Koniec. :-)
	 */
	return 0;
}

void
bold(const char tekst[])
{
	/*
	 * Trzeba co tutaj tumaczy?
	 */
	printf(BOLD "%s" NORMAL, tekst);
}

void
oblicz(u_int data, u_int koniec, u_int *sumimpulsy, double *sumkoszt, char *flagi, const size_t rozm)
{
	/*
	 * Struktura do przechowywania daty.
	 */
	struct tm       atime;

	/*
	 * Do przechowywania pojedyczej flagi.
	 */
	char           *flaga;

	/*
	 * Taki bufor...
	 */
	char            bufor[100];

	/*
	 * Liczba impulsw.
	 */
	u_int	    impulsy = 0;

	/*
	 * Koszt poczenia.
	 */
	double          koszt = 0.00;

	/*
	 * Cena impulsu poza ryczatem.
	 */
	const double    cenaimpulsu = 0.35;

	/*
	 * Dugo impulsu w droszej taryfie.
	 */
	const u_int	dlimpulsudrozej = 180;

	/*
	 * Dugo impulsu w taszej taryfie.
	 */
	const u_int	dlimpulsutaniej = 360;

	/*
	 * Zerujemy zawarto tablicy 'flagi'.
	 */
	bzero(flagi, rozm);

	/*
	 * Dopki 'data' bdzie mniejsza lub rwna 'koniec' (;-) strasznie
	 * trudne).
	 */
	while (data <= koniec) {
		/*
		 * Do bufora zapisujemy dat.
		 */
		snprintf(bufor, sizeof(bufor) - 1, "%u", data);

		/*
		 * I zapisujemy j w strukturze 'atime'.
		 */
		if (strptime(bufor, "%s", &atime) == NULL) {
			fprintf(stderr, "oblicz(): bd strptime()");
			exit(1);
		}

		/*
		 * Jeeli dzie to sobota lub niedziela, czyli weekend, czyli
		 * tasza taryfa, to...
		 */
		if (atime.tm_wday == 0 || atime.tm_wday == 6) {
			/*
			 * Jako flag ustawiamy 'w', zwikszamy liczb impulsw,
			 * zwikszamy cen poczenia, zwikszamy warto 'daty'
			 * o odpowiedni dugo impulsu.
			 */
			flaga = "w";
			impulsy++;
			koszt += cenaimpulsu;
			data += dlimpulsutaniej;
		} else {
			/*
			 * Jeeli to nie jest weekend...
			 */
			if (atime.tm_hour >= 8 && atime.tm_hour < 18) {
				/*
				 * I do tego to jest midzy 8:00 a 18:00, to
				 * musi to by drosza taryfa. Reszt ju
				 * znacie.
				 */
				flaga = "d";
				impulsy++;
				koszt += cenaimpulsu;
				data += dlimpulsudrozej;
			} else if (atime.tm_hour < 8 || atime.tm_hour >= 18) {
				/*
				 * Koniec kocw, jeeli to nie jest weekend i
				 * jest midzy 18:00 a 8:00, to musi to by
				 * tasza taryfa.
				 */
				flaga = "t";
				impulsy++;
				koszt += cenaimpulsu;
				data += dlimpulsutaniej;
			} else {
				/*
				 * Jaki bd w algorytmie?
				 */
				fprintf(stderr, "\nBd. Nie dopasowano taryfy.\n");
				exit(1);
			}
		}

		/*
		 * Maa "kompresja" flag. Jezeli 'flaga' znajduje si ju na
		 * kocu 'flagi', to nie zapisujemy jej n-ty raz. Jeeli w
		 * 'flagi' nie ma jeszcze flag, to dopisujemy j.
		 */
		if (flagi[0] == '\0' || flagi[strlen(flagi) - 1] != *flaga) {
			/*
			 * Jeeli ostatni i przedostatni znak w 'flagi' to '\0',
			 * to znaczy, e zmieci si tam co najmniej jeszcze
			 * jedna flaga. Dopisujemy.
			 */
			if (flagi[rozm - 1] == '\0' && flagi[rozm - 2] == '\0') {
				strcat(flagi, flaga);
			} else {
				/*
				 * Jeeli przedostatni znak w 'flagi' to nie
				 * '\0', to zamieniamy go na '!'. Ten fragment
				 * zostanie wywoany tylko wtedy, gdy we 'flagi'
				 * nie zmieci si ju aden znak.
				 */
				flagi[rozm - 2] = '!';
			}
		}
	}

	/*
	 * Wyiwetlamy liczb impulsw i koszt poczenia.
	 */
	printf("%9u ", impulsy);
	bold("|");
	printf("%9.2f ", koszt);

	/*
	 * Zwikszamy warotci odpowiednich ziennych.
	 */
	*sumimpulsy += impulsy;
	*sumkoszt += koszt;
}

void
version(void)
{
	/*
	 * Hmmm, to pewnie wywietla numer wersji programu, ale do koca pewien
	 * nie jestem ;-).
	 */
	printf("Trf " VERSION " by Maciej Korze <eaquer@ceti.pl>\n");
}

void
usage(void)
{
	/*
	 * Wywietla list argumentw.
	 */
	printf("Uycie: trfs [-V] [-m miesic]\n"
	       "\t-V\t\t\twywietla wersj programu\n"
	       "\t-m miesic\t\tnumer miesica (od 1 do 12)\n");
}

char           *
pobierz(FILE * fd1, FILE * fd2, char *bufor, const int lim)
{
	/*
	 * Czytamy z pierwszego pliku.
	 */
	if (plik == 1) {
		if (fgets(bufor, lim, fd1) != NULL) {
			/*
			 * Jeeli funkcji fgets() udao si odczyta "co", to
			 * zwracamy to.
			 */
			return (bufor);
		} else {
			/*
			 * W przeciwnym wypadku bdziemy czyta z drugiego
			 * pliku.
			 */
			plik = 2;
		}
	}

	/*
	 * Czytamy z drugiego pliku.
	 */
	if (plik == 2) {
		/*
		 * Jeeli moemy odczyta z tego pliku, to...
		 */
		if (fd2 != NULL) {
			/*
			 * Zwracamy to samo, co funkcja fgets().
			 */
			return (fgets(bufor, lim, fd2));
		} else {
			/*
			 * Jeeli nie moemy odczyta z tego pliku, to zwracamy
			 * NULL.
			 */
			return (NULL);
		}
	}

	/*
	 * Ten fragment nie powinien by wykonany.
	 */
	fprintf(stderr, "Bd w funkcji pobierz().\n");
	exit(1);
}

u_int 
validate(const char text[], const char *allowed)
{
	u_int	    i;

	/*
	 * Dla kadego znaku z textu[] sprawdzamy czy znajduje si on w
	 * tablicy allowed_char. Jeeli nie, to robimy return 0 i do
	 * wiedzenia.
	 */
	for (i = 0; text[i] != '\0'; i++) {
		if (strchr(allowed, text[i]) == NULL) {
			return 1;
		}
	}
	return 0;
}

u_int
next_date_start(const char date[]) {
		/*
		 * Liczba odczytanych odstpw (odstpt = nie-spacja + spacja).
		 */
		u_int spaces;

		/*
		 * Znak od ktrego rozpoczyna si nastpna data.
		 */
		u_int date_start;

		for (spaces = 0, date_start = 0 ; spaces < 6 ; date_start++) {
			if ( date[date_start] == ' ' && date[date_start - 1] != ' ') {
				spaces++;
			}
		}

		return date_start;
}

